package cz.cvut.ok.kapo.paa.batoh;

import java.util.Vector;


/**
 * Trida reprezentuje reseni dane konfigurace.
 * @author karel
 *
 */
public class Solution {
	private Configuration conf;
	private Vector<Item> items;
	private long solutionTime;
	private int statesCount;

	/**
	 * Konstruktor. Inicializuje reseni.
	 * @param conf konfigurace, ke ktere se reseni vztahuje.
	 */
	public Solution(Configuration conf) {
		super();
		this.conf=conf;
		this.items = new Vector<Item>(conf.getItemsCount());
		this.solutionTime=0L;
		this.statesCount=0;
	}
	
	/**
	 * @return Returns the statesCount.
	 */
	public int getStatesCount() {
		return statesCount;
	}

	/**
	 * @param statesCount The statesCount to set.
	 */
	public void setStatesCount(int statesCount) {
		this.statesCount = statesCount;
	}

	/**
	 * Prida predmet, ktery patri do batohu.
	 * @param item predmet v batohu.
	 */
	public void addItem(Item item){
		items.add(item);
	}
	
	/**
	 * Cas v nanosekundach
	 * @return Returns the solutionTime.
	 */
	public long getSolutionTime() {
		return solutionTime;
	}

	/**
	 * nastavit cas v nanosekundach
	 * @param solutionTime The solutionTime to set.
	 */
	public void setSolutionTime(long costTime) {
		this.solutionTime = costTime;
	}

	/**
	 * Pocet veci v konfiguraci.
	 */
	public int getItemsCount(){
		return items.size();
	}

	/**
	 * Vrati obsah batohu.
	 * @return vektor predmetu v batohu.
	 */
	public Vector<Item> getItems(){
		return items;
	}
	
	/**
	 * Vrati obsah batohu.
	 * @return vektor predmetu v batohu.
	 */
	public Item getItem(int i){
		return items.get(i);
	}

	/**
	 * Spocita celkovou cenu reseni a vrati ji.
	 * @return celkovou cenu reseni.
	 */
	public int getSolutionCost(){
		int cost=0;
		for(Item tmp:items)
			cost += tmp.getCost();
		
		return cost;
	}

	/**
	 * Spocita celkovou vahu veci reseni a vrati ji.
	 */
	public int getSolutionWeight(){
		int weight=0;
		for(Item tmp:items)
			weight += tmp.getWeight();
		
		return weight;
	}
	
	/**
	 * Vrati konfiguraci pridruzenou k reseni.
	 */
	public Configuration getConfiguration(){
		return conf;
	}
	
	/**
	 * Pretizena pro jednoduche hledani pozadovaneho reseni.
	 */
	@Override public boolean equals(Object o){
		return getConfiguration().getId() == ((Solution)o).getConfiguration().getId();
	}
}
