/**
 * 
 */
package cz.cvut.ok.kapo.paa.batoh;

import java.io.IOException;
import java.io.Writer;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;

/**
 * @author karel
 *
 */
public class Statistic {

	private boolean printHeader;
	
	/**
	 * Predstavuje celkovou strukturu objektu: 
	 * enginy -> jednotlive kapacity batohu -> jednotlive instance
	 */
	private Hashtable<String,Hashtable<Integer,LinkedList<StatItem>>> engines;
	
	/**
	 * 
	 */
	public Statistic(boolean printHeader) {
		super();
		this.engines=new Hashtable<String,Hashtable<Integer,LinkedList<StatItem>>>();
		this.printHeader = printHeader;
	}
	
	/**
	 * Vytahne statistiky patrici danemu enginu a prida do nich 
	 * data z teto solution.
	 * @param solution
	 * @param engineName
	 */
	public void addSolution(Solution solution, String engineName){
		if (solution==null)
			return;
		
		Hashtable<Integer,LinkedList<StatItem>> batohSizes;
		batohSizes = engines.get(engineName);
		if (batohSizes == null)
			batohSizes = new Hashtable<Integer,LinkedList<StatItem>>();
		
		Integer itemsCount = new Integer(solution.getConfiguration().getItemsCount());
		LinkedList<StatItem> items = batohSizes.get(itemsCount); 
		if (items == null)
			items = new LinkedList<StatItem>();
		
		items.add(new StatItem(solution.getConfiguration().getId(), solution.getStatesCount(),solution.getSolutionCost(),solution.getSolutionTime()));
		
		batohSizes.put(itemsCount, items);
		engines.put(engineName,batohSizes);
	}
	
	/**
	 * Zapise statistiky do writeru <b>where</b> tykajici se enginu <b>engine</b>.
	 * @param where kam se zapisi statistiky
	 * @param engine ke kteremu enginu patri statistiky
	 */
	public void writeStats(Writer where, String engineName){
		Hashtable<Integer,LinkedList<StatItem>> batohSizes;
		batohSizes = engines.get(engineName);
		if (batohSizes == null){
			System.out.println("Engine: "+engineName+" neni ve statistikach ulozen!");
			return;
		}
		Enumeration<Integer> bSizes = batohSizes.keys();
		try {
			if (printHeader)
				where.write("pocet predmetu; prumerny pocet stavu; prumerny cas; prumerna cena\r\n");
			
			while (bSizes.hasMoreElements()) {
				Integer count = bSizes.nextElement();
				LinkedList<StatItem> items = batohSizes.get(count);
				StatItem tmp = new StatItem(0,0,0,0.0);
				//spocitame statistiky pres vsechny instance
				for (StatItem si: items)
					tmp.add(si);
				tmp.divide(count.intValue());
				
				where.write(count.intValue()+";"
						+tmp.statesCount+";"
						+tmp.time+";"
						+tmp.statesCost
						+"\r\n");
			}
		}
		catch(IOException ioe){
			System.out.println("Chyba pri zapisu statistik!"+ioe.getMessage());
			ioe.printStackTrace();
		}
	}

	
	/**
	 * Zapise souhrnne statistiky do writeru <b>where</b>, ktere jsou vztazeny
	 * k algoritmu mainEngoneName. Zapisuji se casove  zrychleni a chyba. pro
	 * jednotlive pocty predmetu.
	 * @param where kam se zapisi statistiky
	 * @param engine ke kteremu enginu se vztahuji statistiky
	 */
	public void writeStatsWithErrors(Writer where, String mainEngineName){
		Hashtable<Integer,LinkedList<StatItem>> mainBatohSizes;
		mainBatohSizes = engines.get(mainEngineName);
		if (mainBatohSizes == null){
			System.out.println("Engine: "+mainEngineName+" neni ve statistikach ulozen!");
			return;
		}
		Enumeration<String> engineNames = engines.keys();
		try {
			if (printHeader)
				where.write("algoritmus;pocet predmetu; prumerne zrychleni; prumerna chyba\r\n");
		}catch(IOException ioe){
			System.out.println("Chyba pri zapisu statistik!"+ioe.getMessage());
			ioe.printStackTrace();
			return;
		}
		
		while(engineNames.hasMoreElements()){
			String engineName = engineNames.nextElement();
//			if (mainEngineName.equals(engineName))
	//			continue;
			Hashtable<Integer,LinkedList<StatItem>> engine = engines.get(engineName);
			Enumeration<Integer> bSizes = mainBatohSizes.keys();
			try {
				
				while (bSizes.hasMoreElements()) {
					Integer count = bSizes.nextElement();
					LinkedList<StatItem> mainItems = mainBatohSizes.get(count);
					LinkedList<StatItem> items = engine.get(count);
					if (items == null)
						continue;
					double chyba = 0.0;
					double acceler = 0.0;
					//spocitame statistiky pres vsechny instance
					for (int i=0;i< mainItems.size(); i++){
						if (items.get(i).id!=mainItems.get(i).id)
							continue;
						chyba += items.get(i).statesCost/(double)mainItems.get(i).statesCost;
						System.out.println("BAB:"+mainItems.get(i).time);
						System.out.println("other:"+items.get(i).time);
						double origTime = mainItems.get(i).time;
						acceler += (items.get(i).time)/origTime;
					}
					chyba /= mainItems.size();
					acceler /= mainItems.size();
					where.write(engineName+";"
							+count.intValue()+";"
							+acceler+";"
							+chyba
							+"\r\n");
				}
			}
			catch(IOException ioe){
				System.out.println("Chyba pri zapisu statistik!"+ioe.getMessage());
				ioe.printStackTrace();
			}
		}
	}

	/**
	 * Predstavuje jeden prvek ve strukture statistik
	 * @author karel
	 */
	public class StatItem {
		public int id;
		/**
		 * cas v milisekundach
		 */
		public double time;
		public int statesCount;
		public int statesCost;
		
		public StatItem(int id, int count, int cost, double time){
			this.id=id;
			this.statesCount=count;
			this.time=time;
			this.statesCost = cost;
		}
		
		public void add(StatItem item){
			time += item.time;
			statesCount += item.statesCount;
			statesCost += item.statesCost;
		}
		
		public void divide(int divisor){
			time /= divisor;
			statesCount /= divisor;
			statesCost /= divisor;
		}
	}
}
