package cz.cvut.ok.kapo.paa.batoh.engines;

import java.util.ArrayList;

import cz.cvut.ok.kapo.paa.batoh.Configuration;
import cz.cvut.ok.kapo.paa.batoh.Item;
import cz.cvut.ok.kapo.paa.batoh.Solution;

/**
 * Trida pro vypocet obsazeni batohu metodou hrube sily.
 * 
 * @author karel
 */
public abstract class Engine {
	protected Configuration conf; // konfigurace problemu

	protected Solution sol; // reseni problemu

	protected ArrayList<Item> list; // seznam

	/**
	 * Nastavi konfiguraci a inicializuje objekty.
	 * @param conf konfigurace problemu
	 */
	public Engine(Configuration conf) {
		super();
		initialize(conf);
	}

	/**
	 * Prazdny konstruktor. Kvuli moznosti vytvaret instance 
	 * tridy pres Class.forName().newInstance().
	 */
	public Engine(){
		super();
	}
	
	/**
	 * Provede nastaveni objektu.
	 */
	public void initialize(Configuration conf){
		this.conf = conf;
		this.sol = new Solution(conf);
		this.list = new ArrayList<Item>();
	}
	
	/**
	 * Vrati soucet cen polozek v danem arraylistu.
	 * 
	 * @param items
	 *            seznam polozek.
	 * @return cena polozek.
	 */
	protected int getCost(ArrayList<Item> items) {
		int cost = 0;
		for (Item tmp: items){
			cost += tmp.getCost();
		}
		return cost;
	}

	/**
	 * Vrati soucet vahy polozek v danem arraylistu.
	 * 
	 * @param items
	 *            seznam polozek.
	 * @return vaha polozek.
	 */
	protected int getWeight(ArrayList<Item> items) {
		int weight = 0;
		for (Item tmp:items){
			weight += tmp.getWeight();
		}
		return weight;
	}

	/**
	 * Vrati reseni.
	 */
	public Solution getSolution() {
		return sol;
	}

	/**
	 * Zmeri cas vypoctu daneho reseni.
	 */
	public final void execute(){
		long start,end;
		start = System.nanoTime();
		run();
		end = System.nanoTime();
		getSolution().setSolutionTime(end-start);
		getSolution().setStatesCount(this.getStatesCount());
	}

	/**
	 * Spusti vypocet reseni z obsazene konfigurace.
	 */
	protected abstract void run();
	

	/**
	 * Vrati pocet navstivenych stavu.
	 */
	protected abstract int getStatesCount();

}