package cz.cvut.ok.kapo.paa.batoh.engines;

import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import cz.cvut.ok.kapo.paa.batoh.Configuration;
import cz.cvut.ok.kapo.paa.batoh.Solution;
import cz.cvut.ok.kapo.paa.batoh.utils.ConfigurationException;
import cz.cvut.ok.kapo.paa.batoh.utils.ConfigurationReader;
import cz.cvut.ok.kapo.paa.batoh.utils.SolutionException;
import cz.cvut.ok.kapo.paa.batoh.utils.SolutionsWriter;

/**
 * Hlavni program pro pouziti metody hrube sily.
 * @author karel
 */
public class FileCruncher {
	String engineName;
	Vector<Configuration> confs;
	Vector<Solution> sols;
	
	public FileCruncher(Vector<Configuration> confs, String engineName) {
		super();
		this.confs = confs;
		this.sols = new Vector<Solution>();
		this.engineName = engineName;
	}
	
	/**
	 * Nacte reseni a postupne je v cyklu spusti.
	 *
	 */
	public void execute(){
		Engine engine;
		for (Configuration conf: confs){
			try {
				engine = (Engine)Class.forName(engineName).newInstance();
			}
			catch(ClassNotFoundException cnfe){
				System.out.println("Nenalezena trida enginu: "+ engineName);
				cnfe.printStackTrace();
				continue;
			}
			catch (Exception e){
				e.printStackTrace();
				continue;
			}
			engine.initialize(conf);
			engine.execute();	//spustime vypocet
			sols.add(engine.getSolution());	//ulozime solution
		}
	}
	
	/**
	 * Vrati vsechna reseni, ktera se spocitala jako vektor.
	 */
	public Vector<Solution> getSolutions(){
		return this.sols;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		FileReader fr;
		FileWriter fw;
		ConfigurationReader cr;
		SolutionsWriter sw;
		
		if (args.length<2){
			System.out.println("Chybi parametry: <soubor.vstupni.konfigurace> <soubor.vysputni.konfigurace>");
			return;
		}
		
		try {
			fr = new FileReader(args[0]);
			fw = new FileWriter(args[1]);
		}
		catch (IOException ex){
			System.out.println("Chyba pri otevirani souboru!"+ex.getMessage());
			ex.printStackTrace();
			return;
		}
		cr = new ConfigurationReader(fr);
		try {
			cr.readConfigurations();
		}
		catch(ConfigurationException ex){
			System.out.println("Chyba pri nacitani konfiguraci!!\n\r"+ex.getMessage());
			ex.printStackTrace();
			return;
		}
		FileCruncher obj = new FileCruncher(cr.getConfigurations(),"cz.cvut.ok.kapo.paa.batoh.engines.sh.SimpleHeuristicEngine");
		obj.execute();
		sw = new SolutionsWriter(fw);
		sw.setSolutions(obj.getSolutions());
		obj=null;
		
		try {
			sw.writeSolutios();
		}
		catch(SolutionException ex){
			System.out.println("Chyba pri zapisu reseni!!\n\r"+ex.getMessage());
			ex.printStackTrace();
		}
		try {
			fr.close();
			fw.close();
		}
		catch(IOException ex){
			System.out.println("Chyba pri zavirani souboru!"+ex.getMessage());
			ex.printStackTrace();
			return;
		}
	}

}
