package cz.cvut.ok.kapo.paa.batoh.engines.bab;

import java.util.ArrayList;
import java.util.Arrays;

import cz.cvut.ok.kapo.paa.batoh.Configuration;
import cz.cvut.ok.kapo.paa.batoh.Item;
import cz.cvut.ok.kapo.paa.batoh.engines.Engine;
import cz.cvut.ok.kapo.paa.utils.MyStack;

/**
 * Trida pro vypocet obsazeni batohu metodou vetvi a hranic (Branch And Bounds).
 * @author karel
 */
public class BranchAndBoundsEngine extends Engine{
	protected Item [] items;	//serazene pole veci z batohu
	protected int [] branchCost; // seznam
	protected MyStack<Integer> st;
	
	/**
	 * Inicializuje objekt.
	 * @param conf konfigurace predmetu a batohu.
	 */
	public BranchAndBoundsEngine(Configuration conf) {
		super(conf);
		initialize(conf);
	}

	public BranchAndBoundsEngine(){
		super();
	}
	
	/**
	 * Provede nastaveni objektu.
	 */
	@Override
	public void initialize(Configuration conf){
		super.initialize(conf);
		items = conf.getItems().toArray(new Item[0]);
		branchCost = new int[items.length];
		st = new MyStack<Integer>();
	}

	@Override 
	protected void run(){
		Arrays.sort(items);	//setridime vzestupne
		//vypocteme cenu podvetvi; scitame od konce
		branchCost[branchCost.length-1]=items[branchCost.length-1].getCost();
		for (int i = branchCost.length-2; i >=0 ; i--) {
			branchCost[i]=items[i].getCost();
			branchCost[i]+=branchCost[i+1];
		}

		ArrayList<Item> best=new ArrayList<Item>();
		int bestCost=getCost(best);	//vynulujeme
		int actualCost;
		int idx=0;
		//udrzuje vlastne indexy veci v batohu
		Item itm = items[idx];
		list.add(itm);	
		st.push(new Integer(idx));//vlozime inicialni prvek
		actualCost = itm.getCost();
		while (!list.isEmpty() || itm.compareTo(items[items.length-1])!=0){
			//BaB podminka: Cs + Cb < Cmax, pak continue
			if (idx>0 && idx<items.length-1 && list.size()>0 && actualCost+branchCost[idx+1] < bestCost){
				actualCost -= list.remove(list.size()-1).getCost();
				idx = st.pop().intValue();
				continue;
			}
			if (getWeight(list)>conf.getSize()){
				actualCost -= list.remove(list.size()-1).getCost();
				idx = st.pop().intValue();
				continue;
			}
			if (bestCost<actualCost){//pokud je aktualni cena lepsi
				bestCost=actualCost;
				best=new ArrayList<Item>(list);
			}
			idx++; //posuneme se na dalsi prvek
			if (idx >= items.length){	//pokud jsme vycerpali vsechny prvky
				if (st.isEmpty())//znamena, ze jsme dopocitali
					break;
				idx=st.pop().intValue();
				actualCost -= list.remove(list.size()-1).getCost();
			}else{//vlozime do batohu dalsi vec
				st.push(new Integer(idx));
				list.add(items[idx]);
				actualCost += items[idx].getCost();
			}
		}
		
		for (int i = 0; i<best.size(); i++){	//naplnime reseni
			sol.addItem(best.get(i));
		}
	}

	@Override
	protected int getStatesCount() {
		return st.getPops();
	}
}
