package cz.cvut.ok.kapo.paa.batoh.engines.bch;

import java.util.Arrays;

import cz.cvut.ok.kapo.paa.batoh.Configuration;
import cz.cvut.ok.kapo.paa.batoh.Item;
import cz.cvut.ok.kapo.paa.batoh.engines.Engine;

/**
 * Trida pro vypocet obsazeni batohu metodou nejlepsi ceny.
 * 
 * @author karel
 */
public class BestCostHeuristicEngine extends Engine{

	protected Item[] items; // serazene pole veci z batohu

	/**
	 * Inicializuje objekt, setridi pole.
	 * 
	 * @param conf
	 *            konfigurace predmetu a batohu.
	 */
	public BestCostHeuristicEngine(Configuration conf) {
		super(conf);
		initialize(conf);
	}

	public BestCostHeuristicEngine(){
		super();
	}
	
	/**
	 * Provede nastaveni objektu.
	 */
	@Override
	public void initialize(Configuration conf){
		super.initialize(conf);
		items = new BestCostHeuristicItem[conf.getItemsCount()];
		int i=0;
		for (Item item: conf.getItems()){
			BestCostHeuristicItem tmp = new BestCostHeuristicItem(item);
			items[i++]=tmp;
		}
		Arrays.sort(items); // setridime vzestupne
	}

	/**
	 * Do reseni prida pouze predmet s nejvyssi cenou.
	 * @see cz.cvut.ok.kapo.paa.batoh.engines.sh.Engine#execute()
	 */
	@Override protected void run() {
		int i=0;
		Item tmp;
		tmp = items[i++];
		while (tmp.getWeight() > conf.getSize() || i<items.length){
			tmp = items[i++];
		}
		if (i<=items.length)
			sol.addItem(tmp);
	}

	@Override
	protected int getStatesCount() {
		return 0;
	}

}
