package cz.cvut.ok.kapo.paa.batoh.engines.ch;

import cz.cvut.ok.kapo.paa.batoh.Configuration;
import cz.cvut.ok.kapo.paa.batoh.engines.Engine;
import cz.cvut.ok.kapo.paa.batoh.engines.bch.BestCostHeuristicEngine;
import cz.cvut.ok.kapo.paa.batoh.engines.sh.SimpleHeuristicEngine;

/**
 * Trida pro vypocet obsazeni batohu metodou kombinovane heuristiky.
 * 
 * @author karel
 */
public class CombinedHeuristicEngine extends Engine{
	private BestCostHeuristicEngine engine1;
	private SimpleHeuristicEngine engine2;
	private int statesCount;
	
	/**
	 * Inicializuje objekt, vytvori potrebne enginy.
	 * 
	 * @param conf
	 *            konfigurace predmetu a batohu.
	 */
	public CombinedHeuristicEngine(Configuration conf) {
		super(conf);
		initialize(conf);
	}

	public CombinedHeuristicEngine(){
		super();
	}
	
	/**
	 * Provede nastaveni objektu.
	 */
	@Override
	public void initialize(Configuration conf){
		super.initialize(conf);
		this.engine1 = new BestCostHeuristicEngine(conf);
		this.engine2 = new SimpleHeuristicEngine(conf);
		this.statesCount=0;
	}

	/**
	 * Do reseni prida pouze predmet s nejvyssi cenou.
	 * @see cz.cvut.ok.kapo.paa.batoh.engines.sh.Engine#execute()
	 */
	@Override protected void run() {
		engine1.execute();
		engine2.execute();
		
		if (engine1.getSolution().getSolutionCost()>engine2.getSolution().getSolutionCost()){
			sol=engine1.getSolution();
			System.out.println("BestCostHeuristicEngine");
		}
		else{
			sol=engine2.getSolution();
			System.out.println("SimpleHeuristicEngine");
		}
		statesCount = sol.getStatesCount();
	}

	@Override
	protected int getStatesCount() {
		return statesCount;
	}

}
