package cz.cvut.ok.kapo.paa.batoh.engines.dp;

import cz.cvut.ok.kapo.paa.batoh.Configuration;
import cz.cvut.ok.kapo.paa.batoh.Item;
import cz.cvut.ok.kapo.paa.batoh.engines.Engine;

/**
 * Trida pro vypocet obsazeni batohu metodou dynamickeho programovani.
 * @author karel
 */
public class DynamicProgrammingEngine extends Engine{
	/** Kazdy sloupecek odpovida jednomu predmetu */ 
	private Item [] items;
	/** Sloupecek 1 */
	private DPCell[] column1;
	/** Sloupecek 2 */
	private DPCell[] column2;
	
	/**
	 * Inicializuje objekt.
	 * @param conf konfigurace predmetu a batohu.
	 */
	public DynamicProgrammingEngine(Configuration conf) {
		super(conf);
		initialize(conf);
	}
	
	public DynamicProgrammingEngine(){
		super();
	}

	/**
	 * Provede nastaveni objektu.
	 */
	@Override
	public void initialize(Configuration conf){
		super.initialize(conf);
		items = conf.getItems().toArray(new Item[0]);
		int totalCost = getTotalCost() + 1;
		int totalWeight = getTotalWeight() + 1;
		column1 = new DPCell[totalCost];
		column2 = new DPCell[totalCost];
		//nastavime prvni sloupec
		column1[0] = new DPCell(null,null,0);// nulty radek je vzdy stejny
		column2[0] = new DPCell(null,null,0);
		DPCell endCell = new DPCell(null,null,totalWeight*2);
		for (int i=0; i< column1.length; i++)
			column1[i]=endCell;
	}



	@Override 
	protected void run(){
		DPCell[] last, actual, ctmp;
		actual = column1;
		last = column2;
		int idx=0;
		//pres vsechny sloupecky
		while (idx < items.length){
			//prohodime sloupecky
			ctmp = actual; actual = last; last = ctmp;
			//vycistime actual
			for (int i=1; i<actual.length; i++)
				actual[i]=null;
			//a budeme pocitat nove hodnoty sloupecku actual z hodnot sloupecku
			// last a aktualniho predmetu items[idx]
			// cost oznacuje radek = cenu v batohu
			for (int cost=1; cost<actual.length; cost++){
				int costDiff = cost - items[idx].getCost();
				if (costDiff<0){ // pokud je druhy porovnavany prvek mimo sloupecek, vlozime ze stejne urovne
					actual[cost] = last[cost]; //a nebudou vznikat prazdny bunky :-))
					continue;
				}
				int predValue = last[cost].getWeight();
				int actValue = last[costDiff].getWeight()+items[idx].getWeight();
				if (predValue <= actValue){ //pokud je predchozi bunka s mensi vahou, jen ji zkopirujeme
					actual[cost] = last[cost];
				}else{//jinak vytvorime novou bunku, tj. pridame predmet do batohu
					actual[cost] = new DPCell(last[costDiff], items[idx],actValue);
				}
			}//for () pres vsechny radky sloupecku
			idx++;
		}//while (idx < items.length) pres vsechny sloupecky
		
		//ted jen najit v actual konfiguraci s nejlepsi vahou
		//pujdeme odshora
		int i;
		for (i=actual.length-1; i>=0; i--){
			if (actual[i].getWeight()<= conf.getSize())//konfigurace, kterou uz uneseme
				break;
		}
		//i obsahuje index konfigurace, tak ji jen nasypem do reseni
		DPCell tmp = actual[i];
		while (tmp != null){
			if (tmp.getItm()!=null) // pokud v uzlu je predmet
				sol.addItem(tmp.getItm());//tak ho pridej do reseni
			tmp = tmp.getPrev();
		}
	}
	
	/**
	 * Vrati celkovou cenu vsech veci zadanych v konfiguraci.
	 * @return
	 */
	protected int getTotalCost(){
		int cost = 0;
		for (Item i: items)
			cost += i.getCost();
		return cost;
	}
	
	/**
	 * Vrati celkovou vahu vsech veci zadanych v konfiguraci.
	 * @return
	 */
	protected int getTotalWeight(){
		int weight = 0;
		for (Item i: items)
			weight += i.getWeight();
		return weight;
	}

	@Override
	protected int getStatesCount() {
		return items.length * column1.length;
	}
}
