package cz.cvut.ok.kapo.paa.batoh.utils;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.StringTokenizer;
import java.util.Vector;

import cz.cvut.ok.kapo.paa.batoh.Configuration;

public class ConfigurationReader {
	private BufferedReader fr;
	private Vector<Configuration> confs;
	
	public ConfigurationReader(Reader fr) {
		super();
		this.fr = new BufferedReader(fr);
		this.confs=new Vector<Configuration>();
	}

	/**
	 * Nacte konfiguarace ze vstupniho streamu.
	 * @throws ConfigurationException v pripade ze selze nacitani
	 */
	public void readConfigurations() throws ConfigurationException {
		Configuration conf;
		String line;
		try {
			while ( (line=fr.readLine())!=null){
				StringTokenizer st = new StringTokenizer(line, " ");
				conf = new Configuration();
				try {
					conf.init(st);
				}
				catch(ConfigurationException ex){
					System.out.println("ConfigurationReader.readConfigurations(): Error reading configuration from line:\n\r"+line);
					ex.printStackTrace();
				}
				confs.add(conf);
			}
		}
		catch (IOException e) {
			throw new ConfigurationException("ConfigurationReader.readConfigurations(): Unknown Error!\n\r", e);
		}
	}
	
	/**
	 * Vrati i-tou konfiguraci.
	 * @param i index konfigurace
	 * @return konfigurace
	 */
	public Configuration getConfiguration(int i){
		return confs.get(i);
	}
	
	/**
	 * Vrati vsechny nactene konfigurace.
	 * @return konfigurace
	 */
	public Vector<Configuration> getConfigurations(){
		return confs;
	}
	
	/**
	 * Vrati pocet konfiguraci.
	 * @return pocet konfiguraci
	 */
	public int getConfigurationsCount(){
		return confs.size();
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length<1){
			System.out.println("Chybi argument <nazev.souboru>!");
			return;
		}
		
		try {
			FileReader fr = new FileReader(args[0]);
			ConfigurationReader cr=new ConfigurationReader(fr);
			cr.readConfigurations();
			System.out.println("Pocet konfiguraci : " + cr.getConfigurationsCount());
		}
		catch (Exception e) {
			System.out.print("chyba pri nacitani konfiguraci:"+e.getMessage());
			e.printStackTrace();
		}

	}

}
