package cz.cvut.ok.kapo.paa.batoh.utils;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Vector;

import cz.cvut.ok.kapo.paa.batoh.Item;
import cz.cvut.ok.kapo.paa.batoh.Solution;

/**
 * Trida slouzi pro zapis reseni do vystupniho streamu.
 * @author karel
 */
public class SolutionsWriter {
	private BufferedWriter fw;
	private Vector<Solution> solutions;
	
	public SolutionsWriter(Writer writer) {
		super();
		this.fw=new BufferedWriter(writer);
		this.solutions = new Vector<Solution>();
	}

	/**
	 * Prida toto konkretni reseni mezi vsechna reseni.
	 * @param sol reseni, ktere se ma pridat.
	 */
	public void addSolution(Solution sol){
		solutions.add(sol);
	}

	/**
	 * Nastavi jako reseni parametr.
	 * @param sol reseni, ktere se maji nastavit.
	 */
	public void setSolutions(Vector<Solution> sols){
		this.solutions=sols;
	}
	
	/**
	 * Vraci pocet reseni ulozenych v objektu.
	 */
	public int getSolutionsCount(){
		return solutions.size();
	}
	
	/**
	 * Vrati reseni podle indexu.
	 * @param i index reseni.
	 */
	public Solution getSolution(int i){
		return solutions.get(i);
	}
	
	/**
	 * Zapise vsechna reseni do vystupniho streamu.
	 * @throws SolutionException
	 */
	public void writeSolutios() throws SolutionException{
		for (Solution sol: solutions){
			writeSolution(sol);
			System.out.println(sol.getConfiguration().getId()+" = "+sol.getSolutionTime());
			System.out.flush();
		}
		try  {
			fw.flush();
		} catch (IOException e) {
			throw new SolutionException("SolutionsWriter.writeSolutions(): Chyba pri flushovani bufferu!", e);
		}
	}

	/**
	 * Zapise jedno reseni do vystupniho streamu.
	 * @param sol reseni, ktere se ma zapsat.
	 * @throws SolutionException
	 */
	private void writeSolution(Solution sol) throws SolutionException {
		StringBuffer sb = new StringBuffer();
		sb.append(sol.getConfiguration().getId() + " ");	// ID
		sb.append(sol.getConfiguration().getItemsCount() + " ");	// pocet polozek
		sb.append(sol.getSolutionCost());	// cena reseni
		Vector<Item> sItems = sol.getItems();
		Vector<Item> cItems = sol.getConfiguration().getItems();
		for (Item itm: cItems){
			if (sItems.contains(itm))
				sb.append(" 1");
			else
				sb.append(" 0");
		}
		try {
			fw.write(sb.toString()+"\n\r");
		} catch (IOException e) {
			throw new SolutionException("SolutionsWriter.writeSolution(): Chyba pri zapisu reseni!", e);
		}
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}
